/*
  Dessin d'un cadran gradué circulaire
*/

TCadran = function (params)
{
  const DEMI_PI           = Math.PI / 2;
  const DEUX_PI           = Math.PI * 2;
  const DEB_RAD           = DEMI_PI + Math.PI / 6;
  const FIN_RAD           = DEUX_PI + DEMI_PI - Math.PI / 6;
  const LARGEUR_SECTEUR   = 16;

  var decalage = 20;
  if (this.encadrement == false) decalage = 2;

  this.idCanvas         = params.idCanvas;
  this.dimCanvas        = params.dimCanvas;
  this.mesure           = params.mesure;

  this.rayon            = this.dimCanvas / 2;
  this.centreX          = this.dimCanvas / 2;
  this.centreY          = this.dimCanvas / 2;

  this.clFondCadran     = params.clFondCadran;

  this.encadrement      = params.encadrement;
  this.clBordure        = params.clBordure;
  this.clBiseauExt      = params.clBiseauExt;
  this.clBiseauInt      = params.clBiseauInt;

  this.bandeau          = params.bandeau;
  this.valeurMini       = params.valeurMini;
  this.valeurMaxi       = params.valeurMaxi;
  this.secteurs         = params.secteurs;
  this.clGraduations    = params.clGraduations;
  this.nbPetiteGrad     = params.nbPetiteGrad;
  this.nbMoyenneGrad    = params.nbMoyenneGrad;
  this.nbGrandeGrad     = params.nbGrandeGrad;

  this.policeLegendes   = params.policeLegendes;
  this.ajustLegendX     = params.ajustLegendX;
  this.ajustLegendY     = params.ajustLegendY;

  this.titre            = params.titre;
  this.policeTitre      = params.policeTitre;
  this.posTitre         = params.posTitre;
  this.clTitre          = params.clTitre;

  this.sousTitre        = params.sousTitre;
  this.policeSousTitre  = params.policeSousTitre;
  this.posSousTitre     = params.posSousTitre;
  this.clSousTitre      = params.clSousTitre;

  this.largeurAiguille  = params.largeurAiguille;
  this.clAiguille       = params.clAiguille;
  this.largeurCentre    = params.largeurCentre;
  this.clCentre         = params.clCentre;






  this.dessineFondCadran = function(context)
  {
    context.arc(this.centreX ,this.centreY ,this.rayon, 0, DEUX_PI);
    context.fillStyle = this.clFondCadran;
    context.fill();
  };


  this.dessineEncadrement = function(context)
  {
    context.lineWidth = 2;
    context.beginPath();
    context.arc(this.centreX, this.centreY, this.rayon - 1, 0, DEUX_PI);
    context.strokeStyle = this.clBordure;
    context.stroke();

    context.lineWidth = 8;
    context.beginPath();
    context.arc(this.centreX ,this.centreY ,this.rayon - 6 , 0, DEUX_PI);
    context.strokeStyle = this.clBiseauExt;
    context.stroke();

    context.beginPath();
    context.arc(this.centreX ,this.centreY ,this.rayon - 14 , 0, DEUX_PI);
    context.strokeStyle = this.clBiseauInt;
    context.stroke();
  };


  this.dessineSecteurs = function(context)
  {
    var nbDiv = this.valeurMaxi - this.valeurMini;
    var divRad = (FIN_RAD - DEB_RAD) / nbDiv;
    for (var i = 0; i < this.secteurs.length; i++)
      {
        context.lineWidth = LARGEUR_SECTEUR;
        context.beginPath();
        context.arc(this.centreX, this.centreY, this.rayon - decalage - LARGEUR_SECTEUR / 2 , DEB_RAD + (this.secteurs[i][0] - this.valeurMini) * divRad, DEB_RAD + (this.secteurs[i][1] - this.valeurMini) * divRad);
        context.strokeStyle = this.secteurs[i][2];
        context.stroke();
      }
  };


  this.graduations = function(context, epGrad, nbGrad, lgGrad)
  {
    context.lineWidth = epGrad;
    for (var i = 0; i <= nbGrad; ++i)
    {
      context.beginPath();
      context.strokeStyle = this.clGraduations;
      var angle = (((FIN_RAD - DEB_RAD) / nbGrad) * i) + DEB_RAD;
      context.arc(this.centreX, this.centreY, this.rayon - decalage, angle, angle);
      context.arc(this.centreX, this.centreY, this.rayon - decalage - lgGrad, angle, angle);
      context.closePath();
      context.stroke();
    }
  };


  this.dessineGraduations = function (context)
  {
    this.graduations(context, 1, this.nbPetiteGrad, 7);
    this.graduations(context, 3, this.nbMoyenneGrad, 10);
    this.graduations(context, 3, this.nbGrandeGrad, 15);
  };


  this.dessineLegendes = function (context)
  {
    context.font = this.policeLegendes;
    context.fillStyle = this.clGraduations; // Même couleur que pour les graduations
    for (var i = 0; i <= this.nbGrandeGrad; ++i)
    {
      var angle = (((FIN_RAD - DEB_RAD) / this.nbGrandeGrad) * i) + DEB_RAD;
      var y = this.centreY + Math.sin(angle) * (this.rayon + this.ajustLegendY - decalage - LARGEUR_SECTEUR - 15);
      var x = this.centreX + Math.cos(angle) * (this.rayon + this.ajustLegendX - decalage - LARGEUR_SECTEUR - 15);

      context.textAlign = "center";
      var val = 10 * i + this.valeurMini;
      context.fillText(val.toString(), x, y + 5);
    }
  };


  this.dessineTitre = function(context)
  {
    context.font = this.policeTitre;
    context.fillStyle = this.clTitre;
    context.textBaseline = "middle";
    context.fillText(this.titre, this.centreX, this.posTitre);
  };


  this.dessineSousTitre = function(context)
  {
    context.font = this.policeSousTitre;
    context.fillStyle = this.clSousTitre;
    context.textBaseline = "middle";
    context.fillText(this.sousTitre, this.centreX, this.posSousTitre);
  };


  this.dessineAiguille = function (context)
  {
    var valeur = this.mesure - this.valeurMini;
    context.translate(this.centreX, this.centreY);
    context.rotate((((FIN_RAD - DEB_RAD) / (this.valeurMaxi - this.valeurMini)) * valeur) + DEB_RAD);
    context.beginPath();
    context.arc(0,  0,  this.largeurAiguille, DEMI_PI,  DEMI_PI * 3);
    context.lineTo(this.rayon - decalage - LARGEUR_SECTEUR / 2, 0);
    context.closePath();
    context.fillStyle = this.clAiguille;
    context.fill();
    context.beginPath();
    context.arc(0,  0,  this.largeurCentre, 0,  DEUX_PI);
    context.fillStyle = this.clCentre;
    context.fill();
  };


  this.dessineCadran = function()
  {
    canvas = document.getElementById(this.idCanvas);
    canvas.width = this.dimCanvas;
    canvas.height = this.dimCanvas;

    ctxA = canvas.getContext("2d");
    ctxB = canvas.getContext("2d");

    this.dessineFondCadran(ctxA);
    if (this.encadrement)
    {
      this.dessineEncadrement(ctxA);
    }
    if (this.bandeau)
    {
      this.dessineSecteurs(ctxA);
    }
    this.dessineGraduations(ctxA);
    this.dessineLegendes(ctxA);
    this.dessineTitre(ctxA);
    this.dessineSousTitre(ctxA);
    this.dessineAiguille(ctxB);
  };
};