/***** Include et  déclarations OTA ***********************/
#include <ArduinoOTA.h>
const char* ssid = "id du routeur (box) sur le réseau";
const char* password = "mot de passe du routeur";
/**********************************************************/


// includes et déclarations du sketch

#include "LittleFS.h"
#include "DHT.h"

#define DHTTYPE DHT22

unsigned long reference = 0;
const int ledBuiltIn = 1;
const char brocheDHT = 2;
float hygrometrie = 0.0;
float temperature = 0.0;

DHT capteurDHT(brocheDHT, DHTTYPE);
WiFiServer serveur(80);

void setup()
{
  // Setup du sketch
  pinMode(ledBuiltIn, OUTPUT);
  digitalWrite(ledBuiltIn, HIGH);
  LittleFS.begin();
  capteurDHT.begin();

  /***** Setup OTA ****************************************/
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  while (WiFi.waitForConnectResult() != WL_CONNECTED)
  {
    delay(1000);
    ESP.restart();
  }
  ArduinoOTA.begin();
  /********************************************************/

  serveur.begin();
  // La led clignote 5 fois pour avertir de la fin de la mise à jour
  for (int i = 0; i < 5; i++)
  {
    digitalWrite(ledBuiltIn, LOW);
    delay(200);
    digitalWrite(ledBuiltIn, HIGH);
    delay(200);    
  }
}


void loop()
{
  /***** Procédure OTA ************************************/
  ArduinoOTA.handle();
  /********************************************************/

// Sketch
  /* Récupération des données issues du capteur DHT22 */
  
  unsigned long instant = millis();
  if (instant - reference >= 2000) // Pause entre deux mesures (ce capteur est très lent) 
  {
    hygrometrie = capteurDHT.readHumidity();
    temperature = capteurDHT.readTemperature(); // °C par défaut : (true) pour Fahrenheit
    if (isnan(hygrometrie) || isnan(temperature)) // À tout hasard
    {
      hygrometrie = 0.0;
      temperature = 0.0;
    }
    reference = instant;  
  }

  /* Implémentation du serveur */ 
  
  WiFiClient client = serveur.available();
  if (!client) return;
  client.setTimeout(1000);

  /* Récupération de la requête */ 
  
  String reception = client.readStringUntil('\r'); // On ne lit que la première ligne
  while (client.available()) client.read(); // puis on vide le buffer

  /* Traitement de la requête */  

  if (reception.indexOf("ajax?") != -1) // Si requête ajax
  {
    if (reception.indexOf("maj?") == -1) // Si pas maj
    {
      reception.replace("GET /ajax?", "");
      reception.replace("HTTP/1.1", "");
      reception.trim();
    }
    arHtml(client, "application/json");
    maj(client, hygrometrie, temperature);
  }
  else
  {
    if (reception.startsWith("GET / HTTP/1.1"))
    {
      arHtml(client, "text/html");
      envoiFichier(client, "/meteo.html");
    }
    else if (reception.indexOf("tCadran.js") != -1)
    {
      arHtml(client, "application/javascript");
      envoiFichier(client, "/tCadran.js");
    }
    else if (reception.indexOf("ajax.js") != -1)
    {
      arHtml(client, "application/javascript");
      envoiFichier(client, "/ajax.js");
    } 
    else if (reception.indexOf("thermo.js") != -1)
    {
      arHtml(client, "application/javascript");
      envoiFichier(client, "/thermo.js");
    }
    else if (reception.indexOf("hygro.js") != -1)
    {
      arHtml(client, "application/javascript");
      envoiFichier(client, "/hygro.js");
    }  
    delay(1);
  }
  client.stop();
}

// FONCTIONS

void arHtml(WiFiClient& nomClient, String type)
{
  nomClient.println(F("HTTP/1.1 200 OK"));
  nomClient.println("Content-Type: " + type +"");
  nomClient.println(F("Connection: close"));
  nomClient.println(F(""));
}

void envoiFichier(WiFiClient& nomClient, String fichierEnCours)
{
  
  if (LittleFS.exists(fichierEnCours))
  {
    File fichier = LittleFS.open(fichierEnCours, "r");
    while (fichier.available())
    {
      nomClient.write(fichier.read());
    }
    fichier.close();
  }
  else
  {
    nomClient.println("Page '" + fichierEnCours + "' introuvable...");
  }
}

void maj(WiFiClient& nomClient, float h, float t)
{
  String reponseJSON = "{";
  reponseJSON.concat("\"hygrometrie\":");
  reponseJSON.concat(h);
  reponseJSON.concat(",\"temperature\":");
  reponseJSON.concat(t);  
  reponseJSON.concat("}");
  nomClient.println(reponseJSON);
}
